import React from 'react';
import { Container, Segment, Menu, List, Grid } from 'semantic-ui-react';
import {getRequest} from '../../../actions';
import DevList from './DevList';
import IPMIPage from './IPMIPage';
import DevModal from './Models/DevModal';

const stateDefault = ()=>({
    list: [],
    devSel: {
        id: null,
        type: null,
        data: {
            snmp: [],
            ipmi: []
        }
    },
    devModal: {
        open: false,
        type: 0,
        data: {}
    },
    openedModal: null
})

class ServerPage extends React.Component {
    state = {...stateDefault()}

    componentDidMount(){
        this.getList();
    }

    getList = () => {
        let {showDialog, toggleLoading} = this.props;
        toggleLoading(1);
        fetch('/api/server/getdevicelist', getRequest())
            .then(response => response.json())
            .then(json => {
                toggleLoading(0);
                if(json.status != 1) return showDialog(json.message);
                this.setState({
                    list: json.data.record || []
                })
            })
    }

    selectDevice = (id) => {
        if(!id) return ;
        let {showDialog, toggleLoading} = this.props;
        toggleLoading(1);
        fetch('/api/server/getdevice', getRequest({id}))
            .then(response=>response.json())
            .then(json => {
                toggleLoading(0);
                if(json.status != 1) return showDialog(json.message);
                let ipmi = json.data.rt.ipmi || [];
                let snmp = json.data.rt.snmp || [];

                this.setState({
                    devSel: {
                        ...stateDefault().devSel,
                        id,
                        data: {
                            ipmi,
                            snmp
                        }
                    }
                })
            })
    }
    selectDataType = (type) => {
        this.setState({
            devSel: {
                ...this.state.devSel,
                type: type
            }
        })
    }

    genSubPage = () => {
        let {i18n, showDialog, toggleLoading, callConfirm} = this.props;
        switch(this.state.devSel.type){
            case 'ipmi':
                return <IPMIPage i18n={i18n}
                    showDialog={showDialog}
                    toggleLoading={toggleLoading}
                    callConfirm={callConfirm}
                    list={this.state.devSel.data.ipmi}
                    openModal={this.openModal}
                    closeModal={this.closeModal} />

            default:
                return null;
        }
    }

    openModal = (mtype, type = 0, data = {}) => {
        switch(mtype){
            case 'device':
                this.setState({
                    devModal: {
                        open: true,
                        type,
                        data
                    },
                    openedModal: mtype
                })
                break;
            default: 
                return null;
        }
    }

    closeModal = (mtype) => {
        switch(mtype){
            case 'device':
                this.setState({devModal: {...stateDefault().devModal}, openedModal: null});
                break;
            default: 
                return null;
        }
    }

    submitModal = (type, data) => {
        let open = this.state.openedModal;
        if(open == null) return ;
    }

    render() {
        let {i18n, showDialog, toggleLoading, callConfirm} = this.props;
        return (
            <Container fluid>
                <Grid>
                    <Grid.Column width={3}>
                        <DevList i18n={i18n}
                            toggleLoading={toggleLoading}
                            showDialog={showDialog}
                            callConfirm={callConfirm}
                            sel={this.state.devSel.id}
                            selectDevice={this.selectDevice}
                            list={this.state.list}
                            openModal={this.openModal} />
                    </Grid.Column>
                    <Grid.Column width={13}>
                        {
                            this.state.devSel.id != null ? 
                            (
                                <Container style={{marginBottom: '10px'}}>
                                    <Menu fluid widths={2}>
                                        <Menu.Item name="SNMP" 
                                            active={this.state.devSel.type == 'snmp'} 
                                            onClick={()=>this.selectDataType('snmp')}/>
                                        <Menu.Item name="IPMI" 
                                            active={this.state.devSel.type == 'ipmi'} 
                                            onClick={()=>this.selectDataType('ipmi')}/>
                                    </Menu>
                                </Container>
                            ) : null
                        }
                        {this.genSubPage()}
                    </Grid.Column>
                </Grid>
                <DevModal i18n={i18n}
                    open={this.state.devModal.open}
                    type={this.state.devModal.type}
                    data={this.state.devModal.data}
                    closeModal={this.closeModal} />
            </Container>
        )
    }
}

export default ServerPage;